/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

/**
 * DirNodeEvent
 * 
 * Represents action events in the tree.
 *
 * @version 1.0
 * @author rweltman
 **/
public class DirNodeEvent {

	/**
	 * Constructor
	 *
	 * @param nodes Array of selected tree nodes
	 * @param id Type of action
	 * @param param Additional String information
	 */
    public DirNodeEvent( IDirNode[] nodes, int id, String param ) {
		_nodes = nodes;
		_id = id;
		_param = param;
	}

	/**
	 * Get the nodes selected at the time of the action
	 *
	 * @return The nodes selected at the time of the action
	 */
	public IDirNode[] getNodes() {
		return _nodes;
	}

	/**
	 * Get the ID of the action
	 *
	 * @return The IT of the action
	 */
	public int getID() {
		return _id;
	}

	/**
	 * Get additional info associated with the action
	 *
	 * @return Possibly null info associated with the action
	 */
	public String getParamString() {
		return _param;
	}

	public static final int RUN = 0;

	private IDirNode[] _nodes;
	private int _id;
	private String _param;
}


