/**
 * URL and base64 encoding/decoding functions
 */
import java.util.Hashtable;
import java.util.Enumeration;
import java.text.Collator;

class Mime {
	static final int BACKSLASH = 1;

    Mime() {
	}

	static byte[] decodeBase64(byte[] base64) {
		return decodeBase64(base64, 0, base64.length);
	}

	static byte[] decodeBase64(byte[] base64, int start, int len) {
		byte[] out;
		int n, i, k ;
		long value = 0;
		byte[] buffer;

		if (len == 0)
			return null;

		buffer = new byte[len];

		n = 0;
		for (i = len; i > 3; i -= 4){
			value = 0;
  		    for (int j = 0; j < 4; j++) {
			    byte b, c;

			    b = base64[start + j];
	  		    if (b >= 'A' && b <= 'Z')
				    c = (byte) (b - 'A');
	  		    else if (b >= 'a' && b <= 'z')
				    c = (byte) (b - ('a' - 26));
	  		    else if (b >= '0' && b <= '9')
				    c = (byte) (b - ('0' - 52));
	  		    else if (b == '+')
				    c = 62;
	  		    else if (b == '/')
					c = 63;
	  			else if (b == '=')
					c = 0;
	  			else  {
					return null;
	  			}
	  			value = (value << 6) | c;
			}

			buffer[n++] = (byte) (value >> 16 );
			buffer[n++] = (byte) ((value >> 8) & 0xFF);
			buffer[n++] = (byte) (value & 0xFF);

			start += 4;
		}

		// remove last '\0' from decoded base64 buffer
		k = n - 1;
		while (k >= 0 && buffer[k] == (byte)0)
			k --;
		n = k + 1;

		// add unconverted buffer
		for (k = 0; k < i; k++) {
			buffer[n++] = base64[start + k];

		}
		out = new byte [n];
		for (i = 0; i < n; i++)
			out[i] = buffer[i];

  		return out;
	}

	static byte base64Byte(byte value) {
    	if (value < 0x1A) {
           	return (byte) (value  + (int)'A');
        }
    	else if (value < 0x34)
           	return (byte) (value - 0x1A + (int)'a');
    	else if (value < 0x3E)
           	return (byte) (value - 0x34 + (int)'0');
    	else if (value == 0x3E)
           	return (byte) ('+');
    	else if (value == 0x3F)
           	return (byte) ('/');
    	else
           	return 0;
	}

	static int positive(byte b) {
		int n = b;  // bit binary 
		if (n < 0)
			return 256 + n;
		else
			return n;	
	}
	
	static int positive(char ch) {
		int n = ch;  // bit binary 
		if (n < 0)
			return 0x10000 + n;
		else
			return n;	
	}
	
	static byte[] encodeBase64 (byte in[]) {
		int inPos = 0;
		int outPos = 0;
		long in3;
		byte out[];
		byte src[];

		if ((in.length % 3) != 0)
			out = new byte[4*(in.length/3+1)];
		else
			out = new byte[4*in.length/3];


		// three bytes to four bytes base64 abs
		for (inPos = 0; inPos < in.length; inPos+= 3) {
			if ((inPos + 3) > in.length) {
				int fillZero;
				fillZero = 3 - (in.length - inPos);
				if (fillZero == 1) {
					in3 = (positive(in[inPos]) << 16) +
						(positive(in[inPos+1]) << 8) + 0;				
				}
				else
					in3 = (positive(in[inPos]) << 16) + 0 + 0;
			}
			else {
				in3 = (positive(in[inPos]) << 16) +
					(positive(in[inPos+1]) << 8) + positive(in[inPos+2]);
			}

			for (int i = 0; i < 4; i++) {
				int shift;
				shift = 24 - (i + 1) * 6;
				out[outPos++] = base64Byte((byte) ((in3 >> shift) & 0x3F));
			}
		}
		return out;
	}

	static String encodeModifiedBase64 (char ch) {
		int outPos = 0;
		long in3;
		byte[] out;
		int i, k ;

		out = new byte[4];

		in3 = (long) (positive(ch) << 8);				

		for (i = 0; i < 4; i++) {
			int shift;
			shift = 24 - (i + 1) * 6;
			out[outPos++] = base64Byte((byte) ((in3 >> shift) & 0x3F));
		}

		for (k = i-1; k >= 0; k--) {
			if (out[k] != '=')
				break;
		}

		return new String(out, 0, k);
	}


	static boolean isBase64(String base64) {
		if ((base64.length() % 4) != 0)
			return false;

		for (int i = 0; i < base64.length(); i++) {
			if (base64.charAt(i) > 0x7F)
				return false;
		}

		return true;
	}
	static boolean isSafe(String s) {
		for (int i = 0; i < s.length(); i++) {
			if (s.charAt(i) < 0x20 ||
					s.charAt(i) > 0x80) {
				return false;
	        }
		}

		return true;
	}

	static String escapedUnicodeToText(byte [] escapeUnicode) {
        String text = "";
        int len;
        int i;
        int state = 0;

        len = escapeUnicode.length;
        i = 0;
        while (i < len) {
            if (state == BACKSLASH && escapeUnicode[i] == 'u') {
            	int n = 0;
                int j = 0;
                char ch;
				if ((i+4) < len) {
                	for (j = 0; j < 4; j ++) {
                       	ch = (char) escapeUnicode[i + j + 1];
                       	ch = Character.toUpperCase(ch);
                       	if (ch >= '0' && ch <= '9') {
                           	n = n * 16 +  (ch - '0');
                       	}
                       	else if (ch >= 'A' && ch <= 'F') {
                           	n = n * 16 +  (short) (ch - 'A') + 10;
                       	}
						else  {  // break if not hex value
							break;
						}
                    }
				}

				if (j == 4) {
					if (n == 0x0a) 
						text += "\\n";
					else if (n == 0x0d)
						text += "\\r";
					else 
                   		text += (char) n;
					i += 4;		
				}
				else {
                	text += "\\u";
				}
                state = 0;
            }
            else if (escapeUnicode[i] == '\\') {
                state = BACKSLASH;
            }
            else {
                text += (char) escapeUnicode[i];
            }

            i ++ ;
        }
        return text;
	}

	static String textToEscapedUnicode(String text) {
		if (text == null)
			return null;

		String escapedUnicode = "";
		char ch;
		String zero = "0000";  // reserve 4 spaces

		for (int i = 0; i < text.length(); i++) {
			String hexstr;
			ch = text.charAt(i);
			if ((ch > (char) 0x7F) || (ch < ' ')) {
				hexstr = Integer.toString((short)ch,16);
				if (hexstr.length() < 4) {
					hexstr = zero.substring(0, 4 - hexstr.length()) + hexstr;

				}
              	escapedUnicode += "\\u" + hexstr;
			}
			else
              	escapedUnicode += (char) ch;
		}
		return escapedUnicode;
	}

	static String toHexString(char c) {
		int value;
		value = (short) c;
		if (value < 0)
			value = 0x10000 + value;
    	return Integer.toString(value,16);
	}
	static String toHexString(String s) {
		String hexstr = "";

		for (int i = 0; i < s.length(); i++) {
			int value;
			value = s.charAt(i);
			if (value < 0)
				value = 0x10000 + value;
				
            hexstr += Integer.toString(value,16) + " ";
		}
		return hexstr;
	}

	static String[] sort(Hashtable h) {
		Collator collator = Collator.getInstance();
		String[]  str;
		int i;

		str = new String[h.size()];

		i = 0;
     	for (Enumeration e = h.keys() ; e.hasMoreElements() ; i++) {
			str[i] = (String) e.nextElement(); 
     	}

		for (i = 0; i < str.length-1; i++)	{

			for (int j = i+1; j < str.length; j++) {
				if ( collator.compare(str[i], str[j]) > 0 ) {
					String t = str[i];
					str[i] = str[j];
					str[j] = t;
				}
			}
		}
		return str;
	}

	static boolean isEmptyLine(String line) {
		char ch;
		if (line == null)
			return true;
		for (int i = 0; i < line.length(); i++) {
			ch = line.charAt(i);
			if (Character.isSpaceChar(ch) == false)
				return false;
		}
		return true;
	}
}
