//////////////////////////////////////////////////////
// Teex:   TextEditorXlator
//    Text editor and character set translator
//////////////////////////////////////////////////////

import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

import LdifBufferedReader;

public class Teex extends javax.swing.JPanel implements Serializable {

    /**
     * Blank constructor for deserialization
     */
    public Teex() {
        g_frame = null;
    }

    /**
     * Constructor for initial blank page
     */
    public Teex(javax.swing.JFrame frame, Preferences prefs) {
        this();
        g_frame = frame;
        
        initializePreferences( prefs );

        JPanel panel = createPanels();
        JMenuBar jmb = createMenuBar();
        setLayout(new BorderLayout(0,0));
        add( "North", jmb );
        add( "Center", panel );
    }

    /**
     * Constructor for opening initial file
     */
    public Teex(javax.swing.JFrame frame, String filePath,
                String encoding, Preferences prefs) {
        this(frame, prefs);

        if (encoding != null) {
            g_currentEncoding = encoding;
        }

        File f = new File(filePath);
        openFile(f);
    }

    /**
     * Set the frame. Only used on deserialization.
     *
     * @param frame Frame for panels
     */
    public void setFrame( JFrame frame ) {
        g_frame = frame;
    }

    /**
     * Get the frame.
     *
     * @return Frame for panels
     */
    public JFrame getFrame() {
        return g_frame;
    }

    /**
     * Get the values we need from the preferences object
     *
     * @param prefs An object containing application preferences
     */
    protected void initializePreferences( Preferences prefs ) {
        g_prefs = prefs;
        if ( g_prefs == null ) {
            return;
        }

        String key = "teexbackground";
        try {
            int i = g_prefs.getInteger( key );
            textAreaBackground = new Color( i );
        } catch ( Exception e ) {
            // If the value was not there, stick with the default
            System.out.println( "Teex: no preference " + key );
        }
        key = "teexforeground";
        try {
            int i = g_prefs.getInteger( key );
            textAreaForeground = new Color( i );
        } catch ( Exception e ) {
            System.out.println( "Teex: no preference " + key );
        }
        key = "teextabsize";
        try {
            int i = g_prefs.getInteger( key );
            tabSize = i;
        } catch ( Exception e ) {
            System.out.println( "Teex: no preference " + key );
        }
        key = "teexfontsize";
        try {
            int i = g_prefs.getInteger( key );
            fontSize = i;
        } catch ( Exception e ) {
            System.out.println( "Teex: no preference " + key );
        }
        key = "teexlinewrap";
        try {
            boolean wrap = g_prefs.getBoolean( key );
            lineWrap = wrap;
        } catch ( Exception e ) {
            System.out.println( "Teex: no preference " + key );
        }
    }

    protected JPanel createPanels() {
        JPanel basePanel = new JPanel();
        basePanel.setLayout(new BorderLayout(0,0));

        textScrollPane = new javax.swing.JScrollPane();
        textScrollPane.setVerticalScrollBarPolicy(
            javax.swing.ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
        textScrollPane.setHorizontalScrollBarPolicy(
            javax.swing.ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        textScrollPane.setBounds(0,0,640,480);
        textScrollPane.setFont(dialogFont);
        textScrollPane.setForeground(textAreaForeground);
        textScrollPane.setBackground(textAreaBackground);

        basePanel.add("Center", textScrollPane);

        statusPanel = new javax.swing.JPanel();
        statusPanel.setLayout(new BorderLayout(0,0));
        statusPanel.setBounds(0,577,671,15);
        statusPanel.setFont(dialogFont);
        statusPanel.setForeground(textBackground);
        statusPanel.setBackground(statusBackground);

        basePanel.add("South", statusPanel);

        messageStatus = new javax.swing.JLabel();
        messageStatus.setText("Message:");
        messageStatus.setHorizontalAlignment(
            javax.swing.SwingConstants.CENTER);
        messageStatus.setHorizontalTextPosition(
            javax.swing.SwingConstants.CENTER);
        messageStatus.setBounds(267,0,404,15);
        messageStatus.setFont(dialogBoldFont);
        messageStatus.setForeground(messageForeground);
        messageStatus.setBackground(textBackground);
        statusPanel.add("Center", messageStatus);

        fileEncodingStatus = new javax.swing.JLabel();
        fileEncodingStatus.setHorizontalAlignment(
            javax.swing.SwingConstants.RIGHT);
        fileEncodingStatus.setBounds(0,0,267,15);
        fileEncodingStatus.setFont(dialogBoldFont);
        fileEncodingStatus.setForeground(messageForeground);
        fileEncodingStatus.setBackground(statusBackground);
        statusPanel.add("West", fileEncodingStatus);

        fileEncodingStatus.setText("File: empty    Encoding: " +
                                   g_currentEncoding);

        textArea = new javax.swing.JTextArea();
        textArea.setMargin(new java.awt.Insets(0,0,0,0));
        textArea.setDoubleBuffered(false);
        textArea.setBounds(36,60,565,436);
        textArea.setFont(textFont);
        textArea.setForeground(textAreaForeground);
        textArea.setBackground(textAreaBackground);
        textArea.setTabSize(tabSize);
        textArea.setLineWrap(lineWrap);
        textScrollPane.getViewport().add(textArea);

        return basePanel;
    }

    public void setVisible(boolean b) {
        if (b) {
            setLocation(100, 100);
        }
        super.setVisible(b);
    }

    protected JMenuBar createMenuBar() {
        JMenuBar jmb = new JMenuBar();

        JMenu fileMenu = new JMenu("File");

        JMenuItem openMenu = new JMenuItem("Open...");
        openMenu.addActionListener ( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                if (g_fileChooser == null) {
                    createFileChooser();
                }
                if ( g_fileChooser.showOpenDialog(g_frame) ==
                     JFileChooser.APPROVE_OPTION ) {
                    File f = g_fileChooser.getSelectedFile();
                    openFile(f);
                }
            };
        });

        fileMenu.add(openMenu);

        JMenuItem openInNewWindowMenu =
            new JMenuItem("Open in New Window...");
        openInNewWindowMenu.addActionListener (
            new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                if (g_fileChooser == null) {
                    createFileChooser();
                }
                if (g_fileChooser.showOpenDialog(g_frame) ==
                    JFileChooser.APPROVE_OPTION) {
                    File f = g_fileChooser.getSelectedFile();
                    JFrame frame = new TeexFrame();
                    frame.setTitle("Teex");
                    frame.setVisible(false);
                    frame.setSize(640,480);
                    frame.setBackground(statusBackground);

                    Teex t = new Teex(frame, f.getAbsolutePath(),
                                      g_currentEncoding, g_prefs);
                    frame.setContentPane(t);
                    frame.setVisible(true);
                    g_frameCount++;
                }
            };
        });

        fileMenu.add(openInNewWindowMenu);

        JMenuItem saveMenu = new JMenuItem("Save");
        saveMenu.addActionListener ( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                save();
            }
        });
        fileMenu.add(saveMenu);

        JMenuItem saveAsMenu = new JMenuItem("Save As...");
        saveAsMenu.addActionListener ( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                saveAs();
            }
        });
        fileMenu.add(saveAsMenu);

        JMenuItem exitMenu = new JMenuItem("Exit");
        exitMenu.addActionListener ( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                g_frameCount--;
                if ( g_frameCount <= 0 ) {
                    if ( g_prefs != null ) {
                        g_prefs.save();
                    }
                    g_frame.dispose();
                    System.exit(0);    // close the application
                }
            }
        });
        fileMenu.addSeparator();
        fileMenu.add(exitMenu);

        jmb.add(fileMenu);

        // edit menu
        JMenu editMenu = new JMenu("Edit");

        JMenuItem searchMenu = new JMenuItem("Search...");
        searchMenu.addActionListener (new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                search();
            }
        });
        editMenu.add(searchMenu);

        JMenuItem replaceMenu = new JMenuItem("Replace...");
        replaceMenu.addActionListener ( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                replace();
            }
        });
        editMenu.add(replaceMenu);

        jmb.add(editMenu);

        // view menu
        JMenu viewMenu = new JMenu("View");

        JMenuItem tabMenu = new JMenuItem("Tab Size...");
        tabMenu.addActionListener ( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                setTabs();
            }
        });

        viewMenu.add(tabMenu);

        JMenuItem fontMenu = new JMenuItem("Font Size...");
        fontMenu.addActionListener ( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                setFontSize();
            }
        });
        viewMenu.add(fontMenu);

        JMenuItem colorMenu =
            new JMenuItem("Background Color...");
        colorMenu.addActionListener ( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                textAreaBackground =
                    JColorChooser.showDialog(g_frame,
                                             "Color Chooser",
                                             getBackground());
                textArea.setBackground(textAreaBackground);
                textArea.repaint();
                repaint();
                if ( g_prefs != null ) {
                    g_prefs.setInteger(
                        "teexbackground",
                        textAreaBackground.getRGB()&
                        0x00ffffff );
                }
            };
        });
        viewMenu.add(colorMenu);

        colorMenu = new JMenuItem("Foreground Color...");
        colorMenu.addActionListener ( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                textAreaForeground =
                    JColorChooser.showDialog(g_frame,
                                             "Color Chooser",
                                             getBackground());
                textArea.setForeground(textAreaForeground);
                textArea.repaint();
                repaint();
                if ( g_prefs != null ) {
                    g_prefs.setInteger( "teexforeground",
                                    textAreaForeground.getRGB()&
                                        0x00ffffff );
                }
            };
        });
        viewMenu.add(colorMenu);

        JMenuItem charmapMenu = new JMenuItem("Charmap");
        charmapMenu.addActionListener ( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                (new Charmap("Charmap")).setVisible(true);
            };
        });
        viewMenu.add(charmapMenu);

        wrapMenu = new JMenuItem(lineWrap ? "Do Not Wrap Lines" :
                                 "Wrap Lines");
        wrapMenu.addActionListener (new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                lineWrap = !lineWrap;
                textArea.setLineWrap(lineWrap);
                wrapMenu.setText(lineWrap ? "Do Not Wrap Lines" :
                                 "Wrap Lines");
                if ( g_prefs != null ) {
                    g_prefs.setBoolean( "teexlinewrap", lineWrap );
                }
            };
        });
        viewMenu.add(wrapMenu);
        
        jmb.add(viewMenu);

        // about menu
        JMenu helpMenu = new JMenu("Help");
        JMenuItem aboutMenu = new JMenuItem("About");
        aboutMenu.addActionListener ( new ActionListener() {
            public void actionPerformed( ActionEvent e ) {
                new AboutDialog().show();
            }
        });

        helpMenu.add(aboutMenu);
        jmb.add(helpMenu);

        return jmb;
    }

    public void addNotify() {
        // Record the size of the window prior to calling parent's
        // addNotify.
        Dimension d = getSize();

        super.addNotify();

        if (fComponentsAdjusted)
            return;
        // Adjust components according to the insets
        setSize(getInsets().left + getInsets().right + d.width,
                getInsets().top + getInsets().bottom + d.height);
        Component components[] = getComponents();
        for (int i = 0; i < components.length; i++) {
            Point p = components[i].getLocation();
            p.translate(getInsets().left, getInsets().top);
            components[i].setLocation(p);
        }
        fComponentsAdjusted = true;
    }


    protected void save() {
        if (g_fileName == null) {
            return;
        }

        FileOutputStream fo = null;
        BufferedWriter out;
        try {
            fo = new FileOutputStream(g_fileName);
        }
        catch(Exception fe) {
            messageBox(
                "File output", "Can't open file for writing!");
            return;
        }

        if (fo != null) {
            try {
                if (g_currentEncoding.indexOf("LDIF") != -1) {
                    out = new LdifBufferedWriter(
                        new BufferedWriter (
                            new OutputStreamWriter (fo, "UTF8")));
                } else if (g_currentEncoding.indexOf("EscUnicode") !=
                           -1) {
                    out = new PropertyBufferedWriter(
                        new BufferedWriter (
                            new OutputStreamWriter (fo, "8859_1")));
                } else {
                    out = new BufferedWriter (
                        new OutputStreamWriter (
                            fo, g_currentEncoding));
                }
                textArea.write(out);
                fo.close();
                g_frame.repaint();
            } catch (Exception ioe) {
                messageBox("Java Exception",
                           "please choose a different encoding!");
                System.out.println("Error creating BufferedWriter");
            }
        }
    }


    protected void saveAs() {
        if (g_fileChooser == null) {
            createFileChooser();
        }
        if (g_fileChooser.showSaveDialog(g_frame) ==
            JFileChooser.APPROVE_OPTION) {
            File f = g_fileChooser.getSelectedFile();
            FileOutputStream fo = null;
            BufferedWriter out;

            try {
                fo = new FileOutputStream(f);
            } catch(Exception fe) {
                messageBox("File save",
                           "Can't open file for writing");
                return;
            }

            g_frame.setTitle(f.getPath());
            g_fileName = f.getPath();
            fileEncodingStatus.setText("File: " + f.getName() +
                                       "    Encoding: " +
                                       g_currentEncoding);

            statusPanel.doLayout();

            if ( fo != null ) {
                try {
                    if (g_currentEncoding.indexOf("LDIF") != -1)
                        out = new LdifBufferedWriter(
                            new BufferedWriter (
                                new OutputStreamWriter (fo, "UTF8")));
                    else if ( g_currentEncoding.indexOf("EscUnicode")
                              != -1 ) {
                        out = new PropertyBufferedWriter(
                            new BufferedWriter (
                                new OutputStreamWriter (fo, "8859_1")));
                    } else {
                        out = new BufferedWriter (
                            new OutputStreamWriter (fo,
                                                    g_currentEncoding));
                    }
                    g_frame.setTitle(f.getPath());
                    textArea.write(out);
                    fo.close();
                    repaint();
                } catch (Exception ioe) {
                    messageBox("Java Exception",
                               "please choose a different encoding!");
                    System.out.println("Error creating BufferedWriter");
                }
            }
        }
    }


    protected void search() {
        if ( g_searchDialog == null ) {
            JLabel  jLabel1;
            JButton closeButton;
            g_searchDialog = new JDialog(g_frame);

            g_searchDialog.getContentPane().setLayout(null);
            g_searchDialog.getContentPane().setVisible(true);
            g_searchDialog.setSize(473,80);
            g_searchDialog.setLocation(100,100);

            jLabel1 = new javax.swing.JLabel();
            jLabel1.setText("Search:");
            jLabel1.setBounds(12,12,60,32);
            jLabel1.setFont(dialogBoldFont);
            jLabel1.setForeground(messageForeground);
            jLabel1.setBackground(statusBackground);

            g_searchDialog.getContentPane().add(jLabel1);
            g_searchField = new javax.swing.JTextField();
            g_searchField.setBounds(72,12,202,buttonHeight);
            g_searchField.setFont(searchFont);
            g_searchField.setForeground(textForeground);
            g_searchField.setBackground(textBackground);
            g_searchDialog.getContentPane().add(g_searchField);

            g_findButton = new javax.swing.JButton();
            g_findButton.setText("Find");
            g_findButton.setBounds(300,12,72,buttonHeight);
            g_findButton.setFont(dialogBoldFont);
            g_findButton.setForeground(textForeground);
            g_findButton.setBackground(statusBackground);
            g_searchDialog.getContentPane().add(g_findButton);

            closeButton = new javax.swing.JButton();
            closeButton.setText("Close");
            closeButton.setBounds(384,12,72,buttonHeight);
            closeButton.setFont(dialogBoldFont);
            closeButton.setForeground(textForeground);
            closeButton.setBackground(statusBackground);
            closeButton.addActionListener ( new ActionListener() {
                public void actionPerformed( ActionEvent e ) {
                    messageStatus.setText("Message:");
                    g_searchDialog.setVisible(false); // hide the Frame
                }
            });

            g_searchDialog.getContentPane().add(closeButton);

            g_findButton.addActionListener ( new ActionListener() {
                public void actionPerformed( ActionEvent e ) {
                    String searchText = g_searchField.getText();
                    messageStatus.setText("Message:");
                    if (searchText.length() > 0) {
                        documentSearch(searchText);
                    }
                    else {
                        messageBox("Search",
                                   "No search text specified");
                    }
                };
            });
        }

        String selection = textArea.getSelectedText();
        if (selection != null) {
            g_searchField.setText(selection);
        }

        g_searchStart = textArea.getCaretPosition();
        g_searchDialog.show();

        repaint();
    }


    protected void replace() {
        if ( g_replaceDialog == null ) {
            JLabel jLabel1, jLabel2;
            JButton replaceButton;
            JButton replaceAllButton;
            JButton closeButton;

            g_replaceDialog = new JDialog(g_frame);
            g_replaceDialog.getContentPane().setLayout(null);
            g_replaceDialog.getContentPane().setVisible(true);
            g_replaceDialog.setSize(418,200);
            g_replaceDialog.setLocation(100,100);

            g_replaceSearchField = new javax.swing.JTextField();
            g_replaceSearchField.setBounds(144,24,240,buttonHeight);
            g_replaceSearchField.setFont(searchFont);
            g_replaceSearchField.setForeground(textForeground);
            g_replaceSearchField.setBackground(textBackground);
            g_replaceDialog.getContentPane().add(g_replaceSearchField);

            jLabel1 = new javax.swing.JLabel();
            jLabel1.setText("Search:");
            jLabel1.setBounds(36,24,84,28);
            jLabel1.setFont(dialogBoldFont);
            jLabel1.setForeground(messageForeground);
            jLabel1.setBackground(statusBackground);
            g_replaceDialog.getContentPane().add(jLabel1);

            jLabel2 = new javax.swing.JLabel();
            jLabel2.setText("Replace with:");
            jLabel2.setBounds(36,72,84,36);
            jLabel2.setFont(dialogBoldFont);
            jLabel2.setForeground(messageForeground);
            jLabel2.setBackground(statusBackground);
            g_replaceDialog.getContentPane().add(jLabel2);

            g_replaceField = new javax.swing.JTextField();
            g_replaceField.setBounds(144,72,240,buttonHeight);
            g_replaceField.setFont(searchFont);
            g_replaceField.setForeground(textForeground);
            g_replaceField.setBackground(textBackground);
            g_replaceDialog.getContentPane().add(g_replaceField);

            replaceButton = new javax.swing.JButton();
            replaceButton.setText("Replace");
            replaceButton.setBounds(36,132,108,buttonHeight);
            replaceButton.setFont(dialogBoldFont);
            replaceButton.setForeground(textForeground);
            replaceButton.setBackground(statusBackground);
            g_replaceDialog.getContentPane().add(replaceButton);

            replaceAllButton = new javax.swing.JButton();
            replaceAllButton.setText("Replace All");
            replaceAllButton.setBounds(156,132,108,buttonHeight);
            replaceAllButton.setFont(dialogBoldFont);
            replaceAllButton.setForeground(textForeground);
            replaceAllButton.setBackground(statusBackground);
            g_replaceDialog.getContentPane().add(replaceAllButton);

            closeButton = new javax.swing.JButton();
            closeButton.setText("Close");
            closeButton.setBounds(276,132,108,buttonHeight);
            closeButton.setFont(dialogBoldFont);
            closeButton.setForeground(textForeground);
            closeButton.setBackground(statusBackground);
            g_replaceDialog.getContentPane().add(closeButton);

            replaceButton.addActionListener (new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    String searchText =
                        g_replaceSearchField.getText();
                    String replaceText = g_replaceField.getText();
                    messageStatus.setText("Message:");

                    // get selection,
                    // if selection == searchText, replace it
                    // otherwise search and highlight text
                    if (searchText.length() > 0) {
                        documentReplace(searchText, replaceText);
                    }
                    else {
                        messageBox("Search and Replace",
                                   "Search text is empty");
                    }
                };
            });

            replaceAllButton.addActionListener (new ActionListener() {
                public void actionPerformed( ActionEvent e ) {
                    String searchText =
                        g_replaceSearchField.getText();
                    String replaceText = g_replaceField.getText();

                    // get selection,
                    // if selection == searchText, replace it
                    // otherwise search and highlight text
                    if (searchText.length() > 0) {
                        while(documentReplace(searchText, replaceText)
                              == true);
                    }
                };
            });

            closeButton.addActionListener ( new ActionListener() {
                public void actionPerformed( ActionEvent e ) {
                    messageStatus.setText("Message:");
                    // hide the Frame
                    g_replaceDialog.setVisible(false);
                }
            });

            g_searchStart = textArea.getCaretPosition();
        }

        String selection = textArea.getSelectedText();
        if (selection != null) {
            g_replaceSearchField.setText(selection);
        }

        g_replaceDialog.show();

        messageStatus.setText("Message:");
        statusPanel.doLayout();
        repaint();
    }


    protected void setTabs() {
        if ( g_tabDialog == null ) {
            JLabel  jLabel1;
            JButton closeButton;
            g_tabDialog = new JDialog(g_frame);

            g_tabDialog.getContentPane().setLayout(null);
            g_tabDialog.getContentPane().setVisible(true);
            g_tabDialog.setSize(225,24+2*buttonHeight+24);
            g_tabDialog.setLocation(100,100);

            jLabel1 = new javax.swing.JLabel();
            jLabel1.setText("Tab size:");
            jLabel1.setBounds(12,12,60,32);
            jLabel1.setFont(dialogBoldFont);
            jLabel1.setForeground(messageForeground);
            jLabel1.setBackground(statusBackground);

            g_tabDialog.getContentPane().add(jLabel1);
            g_tabField = new javax.swing.JTextField();
            g_tabField.setText(Integer.toString(tabSize));
            g_tabField.setBounds(72,12,32,buttonHeight);
            g_tabField.setFont(searchFont);
            g_tabField.setForeground(textForeground);
            g_tabField.setBackground(textBackground);
            g_tabDialog.getContentPane().add(g_tabField);

            JButton okButton = new javax.swing.JButton();
            okButton.setText("OK");
            okButton.setBounds(12,12+buttonHeight+5,
                               96,buttonHeight);
            okButton.setFont(dialogBoldFont);
            okButton.setForeground(textForeground);
            okButton.setBackground(statusBackground);
            g_tabDialog.getContentPane().add(okButton);

            closeButton = new javax.swing.JButton();
            closeButton.setText("Cancel");
            closeButton.setBounds(115,12+buttonHeight+5,
                                  96,buttonHeight);
            closeButton.setFont(dialogBoldFont);
            closeButton.setForeground(textForeground);
            closeButton.setBackground(statusBackground);
            closeButton.addActionListener ( new ActionListener() {
                public void actionPerformed( ActionEvent e ) {
                    messageStatus.setText("Message:");
                    g_tabDialog.setVisible(false);
                }
            });

            g_tabDialog.getContentPane().add(closeButton);

            okButton.addActionListener ( new ActionListener() {
                public void actionPerformed( ActionEvent e ) {
                    messageStatus.setText("Message:");
                    String text = g_tabField.getText();
                    if (text.length() > 0) {
                        try {
                            String t = textArea.getText();
                            tabSize = Integer.parseInt(text);
                            textArea.setTabSize(tabSize);
                            textArea.setText( t );
                            if ( g_prefs != null ) {
                                g_prefs.setInteger( "teextabsize",
                                                    tabSize );
                            }
                            textArea.repaint();
                        } catch ( Exception ex ) {
                        }
                        g_tabDialog.setVisible(false);
                    }
                }
            });
        }

        g_tabDialog.show();
        repaint();
    }


    protected void setFontSize() {
        if (g_fontDialog == null) {
            JLabel  jLabel1;
            JButton closeButton;
            g_fontDialog = new JDialog(g_frame);

            g_fontDialog.getContentPane().setLayout(null);
            g_fontDialog.getContentPane().setVisible(true);
            g_fontDialog.setSize(225,24+2*buttonHeight+24);
            g_fontDialog.setLocation(100,100);

            jLabel1 = new javax.swing.JLabel();
            jLabel1.setText("Font size:");
            jLabel1.setBounds(12,12,60,32);
            jLabel1.setFont(dialogBoldFont);
            jLabel1.setForeground(messageForeground);
            jLabel1.setBackground(statusBackground);

            g_fontDialog.getContentPane().add(jLabel1);
            g_fontField = new javax.swing.JTextField();
            g_fontField.setText(Integer.toString(fontSize));
            g_fontField.setBounds(72,12,32,buttonHeight);
            g_fontField.setFont(searchFont);
            g_fontField.setForeground(textForeground);
            g_fontField.setBackground(textBackground);
            g_fontDialog.getContentPane().add(g_fontField);

            JButton okButton = new javax.swing.JButton();
            okButton.setText("OK");
            okButton.setBounds(12,12+buttonHeight+5,
                               96,buttonHeight);
            okButton.setFont(dialogBoldFont);
            okButton.setForeground(textForeground);
            okButton.setBackground(statusBackground);
            g_fontDialog.getContentPane().add(okButton);

            closeButton = new javax.swing.JButton();
            closeButton.setText("Cancel");
            closeButton.setBounds(115,12+buttonHeight+5,
                                  96,buttonHeight);
            closeButton.setFont(dialogBoldFont);
            closeButton.setForeground(textForeground);
            closeButton.setBackground(statusBackground);
            closeButton.addActionListener ( new ActionListener() {
                public void actionPerformed( ActionEvent e ) {
                    messageStatus.setText("Message:");
                    g_fontDialog.setVisible(false);
                }
            });

            g_fontDialog.getContentPane().add(closeButton);

            okButton.addActionListener ( new ActionListener() {
                public void actionPerformed( ActionEvent e ) {
                    messageStatus.setText("Message:");
                    String text = g_fontField.getText();
                    if (text.length() > 0) {
                        try {
                            fontSize = Integer.parseInt(text);
                            Font f = new Font( textFont.getName(),
                                               textFont.getStyle(),
                                               fontSize );
                            textArea.setFont( f );
                            if ( g_prefs != null ) {
                                g_prefs.setInteger( "teexfontsize",
                                                    fontSize );
                            }
                            textArea.repaint();
                        } catch ( Exception ex ) {
                        }
                        g_fontDialog.setVisible(false);
                    }
                }
            });
        }

        g_fontDialog.show();
        repaint();
    }


    void createFileChooser() {
        g_listBox = new JList(g_encoding);

        g_listBox.addListSelectionListener(
            new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent le) {
                g_currentEncoding =
                    (String) g_listBox.getSelectedValue();
            }
        });

        g_listBox.setSelectedIndex(1);
        g_listBox.setPreferredSize(new Dimension(100,30));
        g_listBox.setForeground(textForeground);
        g_listBox.setBackground(Color.gray);

        g_fileChooser = new JFileChooser();
        g_fileChooser.setAccessory(g_listBox);
    }

    void messageBox(String title, String text) {
        if (g_messageDialog == null) {
            JLabel gifLabel;
            JButton okButton;

            g_messageDialog = new JDialog(g_frame);

            g_messageDialog.getContentPane().setLayout(null);
            g_messageDialog.getContentPane().setVisible(true);
            g_messageDialog.setSize(492,195);
            g_messageDialog.setLocation(100,100);

            g_msgText1 = new javax.swing.JLabel();
            g_msgText1.setBounds(180,24,276,24);
            g_msgText1.setFont(dialogBoldFont);
            g_msgText1.setForeground(messageForeground);
            g_msgText1.setBackground(statusBackground);
            g_messageDialog.getContentPane().add(g_msgText1);

            gifLabel = new javax.swing.JLabel();

            gifLabel.setIcon(new RemoteImage("wise-sm.gif"));
            gifLabel.setBounds(12,12,144,144);
            gifLabel.setFont(dialogBoldFont);
            gifLabel.setForeground(messageForeground);
            gifLabel.setBackground(statusBackground);
            g_messageDialog.getContentPane().add(gifLabel);


            g_msgText2 = new javax.swing.JLabel();
            g_msgText2.setBounds(180,60,280,25);
            g_msgText2.setFont(dialogBoldFont);
            g_msgText2.setForeground(messageForeground);
            g_msgText2.setBackground(statusBackground);
            g_messageDialog.getContentPane().add(g_msgText2);

            okButton = new javax.swing.JButton();
            okButton.setText("OK");
            okButton.setBounds(228,108,108,buttonHeight);
            okButton.setFont(dialogBoldFont);
            okButton.setForeground(textForeground);
            okButton.setBackground(statusBackground);


            okButton.addActionListener (new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    g_messageDialog.setVisible(false);
                }
            });

            g_messageDialog.getContentPane().add(okButton);
        }
        g_messageDialog.setTitle(title);
        g_msgText1.setText(text);
        g_messageDialog.show();
    }

    boolean documentSearch(String searchText) {
        int n;
        javax.swing.text.Document doc = textArea.getDocument();

        try {
            g_text = doc.getText(0, doc.getLength());
        }
        catch (Exception doc_e) {
            messageBox("Search and Replace",
                       "Can't grab document text");
            g_text = null;
            return false;
        }

        if ((n = g_text.indexOf(searchText, g_searchStart)) != -1) {
            textArea.select(n, n + searchText.length());
            g_searchStart = n + searchText.length();
            return true;
        }
        else {
            messageStatus.setText("Message: reached end of document");
            repaint();
            g_searchStart = 0;
            return false;
        }
    }

    // true: no more replacement
    // false: done.
    boolean documentReplace(String searchText, String replaceText) {
        String selection;

        selection = textArea.getSelectedText();
        if (selection == null) {
            return documentSearch(searchText);
        }
        else {
            // get current high
            g_searchStart = textArea.getSelectionStart();
            textArea.replaceSelection(replaceText);
            g_searchStart += replaceText.length();
            // move and highlight next item
            return documentSearch(searchText);
        }
    }

    void openFile(File f) {
        textArea.setText("");
        textArea.repaint();

        FileInputStream fi = null;
        BufferedReader in = null;


        try {
            fi = new FileInputStream(f);
        }
        catch(Exception fe) {
            messageBox("File Open", "Can't open file for reading");
            return;
        }
        g_frame.setTitle(f.getPath());
        g_fileName = f.getPath();
        fileEncodingStatus.setText("File: " + f.getName() +
            "    Encoding: " + g_currentEncoding);
        statusPanel.doLayout();

        try {
            if (g_currentEncoding.compareTo("LDIF") == 0) {
                in = new LdifBufferedReader(
                    new BufferedReader (
                        new InputStreamReader (fi, "UTF8")));
            } else if (g_currentEncoding.compareTo("EscUnicode")
                       == 0) {
                in = new PropertyBufferedReader(
                    new BufferedReader (
                        new InputStreamReader (fi, "8859_1")));
            } else {
                in = new BufferedReader (
                    new InputStreamReader (fi, g_currentEncoding));
            }
        } catch (Exception ioe) {
            System.out.println("Exception: can not open " +
                               "LdifBufferedReader with UTF8");
            in = new BufferedReader (new InputStreamReader (fi));
        }

        try {
            textArea.read(in, null);
            in.close();
        } catch (Exception be) {
            messageBox("Document Read",
                       "Document class: read error!");
        }

        repaint();
    }

    static public void displayUsage(boolean showEncoding) {
        System.out.println("Usage: java Teex [filename] " +
                           "[-h] [-encoding encoding] " +
                           "[-u URL -D userID -w password]");
        System.out.println("  For example:");
        System.out.println("  java Teex -u " +
                           "\"ldap://localhost:389/o=airius.com\" " +
                           "-D dakers -w integument");
        System.out.println("  java Teex");
        System.out.println("  java Teex -encoding SJIS notes.sjs");
        System.out.println("  java Teex -encoding LDIF jp.ldif");
        if (showEncoding == true) {
            System.out.println("    Supported encodings:");
            for (int i = 0; i < defaultEncodingList.length; i++) {
                System.out.println("        " +
                                   defaultEncodingList[i]);
            }
        }
    }

    /**
     * Run Teex as an application
     *
     * @param args [-encoding ENC] -u LDAPURL -D UID -w password
     * [-h] [FILE]
     */
    static public void main(String args[]) {
        int i = 0;
        String filePath = null;
        String encoding = null;
        String url = null;
        String uid = null;
        String password = null;

        while (i < args.length) {
            if (args[i].compareTo("-encoding") == 0) {
                if ((i + 1) < args.length) {
                    encoding = args[i+1];
                    i ++;
                }
            } else if (args[i].compareTo("-u") == 0) {
                if ((i + 1) < args.length) {
                    url = args[i+1];
                    i ++;
                }
            } else if (args[i].compareTo("-D") == 0) {
                if ((i + 1) < args.length) {
                    uid = args[i+1];
                    i ++;
                }
            } else if (args[i].compareTo("-w") == 0) {
                if ((i + 1) < args.length) {
                    password = args[i+1];
                    i ++;
                }
            } else if (args[i].compareTo("-h") == 0) {
                displayUsage(true);
                System.exit(0);
            } else if (args[i].startsWith("-") == true) {
                System.out.println("Invalid option: " + args[i]);
                displayUsage(false);
                System.exit(0);
            } else {
                filePath = args[i];
            }
            i ++;
        }

        // setup global encoding list.
        int len = defaultEncodingList.length;
        int newLen = len;
        if (encoding != null) {
            // Check if encoding is valid
            try {
                byte b[] = "abc".getBytes(encoding);
            }
            catch (Exception e) {
                System.out.println(
                    "Error: specified encoding is not supported - " +
                    encoding);
                displayUsage(true);
                System.exit(1);
            }
            newLen++;
            g_encoding = new String[newLen];
            for (i = 0; i < len; i++) {
                g_encoding[i] = defaultEncodingList[i];
            }
            g_encoding[i] = encoding;
        } else {
            g_encoding = defaultEncodingList;
        }

        // Check if file name is valid
        if (filePath != null) {
            File f = new File(filePath);
            if (f.exists() == true) {
                if (encoding != null) {
                    // test to see if new encoding is supported or not
                }
            } else {
                System.out.println("Error: specified file not " +
                                   "found - " + filePath);
                displayUsage(false);
                System.exit(1);
            }
        }

        JFrame frame = new TeexFrame();
        frame.setTitle("Teex");
        frame.setVisible(false);
        frame.setSize(640,480);
        frame.setBackground(textBackground);

        Preferences prefs = null;
        if ( (url != null) && (password != null) && (uid != null) ) {
            prefs = new LDAPPreferences1( url, uid, password );
        }

        Teex t;
        if ( filePath != null ) {
            t = new Teex(frame, filePath, encoding, prefs);
        } else {
            t = new Teex(frame, prefs);
        }

        frame.setContentPane(t);
        frame.setVisible(true);
    }

    // Used for addNotify check.
    boolean fComponentsAdjusted = false;

    javax.swing.JScrollPane textScrollPane;
    javax.swing.JPanel statusPanel;
    javax.swing.JLabel messageStatus;
    javax.swing.JLabel fileEncodingStatus;
    javax.swing.JTextArea textArea;

    static int g_frameCount = 0;
    javax.swing.JFrame g_frame;
    JFileChooser g_fileChooser = null;
    JList g_listBox = null;
    int g_totalBytes = 0;
    String g_textBuffer = "";

    static final String[] defaultEncodingList =
    {"UTF8",   "8859_1",    "SJIS",
     "EUC_JP", "ISO2022JP", "Big5",
     "GBK",    "LDIF",      "EscUnicode"};

    static String[] g_encoding = null;
    String g_currentEncoding = "UTF8";
    String g_fileName = null;

    JDialog g_searchDialog = null;
    JTextField g_searchField = null;
    JButton g_findButton = null;
    int g_searchStart = 0;
    String g_text = null;

    JDialog g_replaceDialog = null;
    JTextField g_replaceSearchField = null;
    JTextField g_replaceField = null;

    JDialog g_tabDialog = null;
    JTextField g_tabField = null;

    JDialog g_fontDialog = null;
    JTextField g_fontField = null;

    JDialog g_messageDialog = null;
    JLabel g_msgText1 = null;
    JLabel g_msgText2 = null;
    JLabel g_msgText3 = null;

    JDialog g_colorDialog = null;

    Preferences g_prefs = null;

    JMenuItem wrapMenu;

    static final int defaultTabSize = 4;
    static final int defaultFontSize = 12;

    static final Color textForeground = new Color(0);
    static final Color textBackground = new Color(16777215);
    static final Color statusBackground = new Color(-3355444);
    static final Color messageForeground = new Color(-10066279);

    static final Font dialogFont = new Font("Dialog", Font.PLAIN,
                                            defaultFontSize);
    static final Font dialogBoldFont = new Font("Dialog", Font.BOLD,
                                                defaultFontSize);
    static final Font textFont = new Font("MonoSpaced", Font.PLAIN,
                                          defaultFontSize);
    static final Font searchFont = new Font("SansSerif", Font.PLAIN,
                                            defaultFontSize);

    static final int buttonHeight = 28;


    // Preferences
    Color textAreaBackground = textBackground;
    Color textAreaForeground = textForeground;
    Font textAreaFont = textFont;
    int tabSize = defaultTabSize;
    int fontSize = defaultFontSize;
    boolean lineWrap = false;
}

class TeexFrame extends JFrame {
    TeexFrame() {
        super();
        addWindowListener(new WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent event) {
                setVisible(false); // hide the Frame
                dispose();       // free the system resources
                System.exit(0);    // close the application
            }
        });
    }
}
