/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import javax.swing.Icon;
import netscape.ldap.LDAPConnection;
import netscape.ldap.LDAPEntry;

/**
 *  Netscape Directory Server 4.0 Directory Entry interface.
 *
 * @author  rweltman
 * @version %I%, %G%
 */

public interface IDirNode extends javax.swing.tree.TreeNode,
                                  javax.swing.tree.MutableTreeNode {

    /**
      * Specifies the name for this object, displayed in tree, right of icon.
      *
      * @return a string representing the object's name
      */
    public String getName();

    /**
     * Specifies an icon for this object, displayed in tree, left of name.
     * The recommended size for this icon is 16x16 pixels.
     *
     * @return an icon representing the object's icon
     */
    public Icon getIcon();

    /**
     * Get the DN of the entry corresponding to this node
     *
     * @return the DN of the node
     */
    public String getDN();

    /**
     * Set the DN of the node
     *
     * @param dn the new DN of the node
     */
    public void setDN( String dn );

    /**
     * Report the entry associated with this node. If the entry has not been
     * retrieved from the Directory yet, it is done now.
     *
     * @return the entry associated with this node. Only a few attributes are
     * retrieved in the entry.
     */
    public LDAPEntry getEntry();

    /**
     * Set the entry for this node
     *
     * @param entry the new entry. May be null to force reinitialization.
     */
    public void setEntry( LDAPEntry entry );

    /**
     * Returns true if the node has read its entry from the Directory
     *
     * @return true if the node has read its entry from the Directory
     */
    public boolean isLoaded();

    /**
     *  Create all the one level depth child nodes
     */
    public void reload();

    /**
     * Initialize the node from data in an entry
     *
     * @param entry An entry initialized with data
     */
    public void initializeFromEntry( LDAPEntry findEntry );

    /**
     *  Check if there are children to this node.
     */
    public void load();

    /**
     * Report if this node is considered a container. This is true if it is
     * one of a defined list of objectclasses, or if it has children.
     *
     * @return true if the node is considered a container.
     */
    public boolean isContainer();
}

