/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.awt.*;
import java.awt.event.*;
import java.util.Enumeration;
import java.util.Hashtable;
import javax.swing.*;
import javax.swing.border.*;

/**
 * SimpleDialog
 * A modeless dialog to launch a panel in the middle of the
 * screen
 *
 * @version 1.0
 * @author rweltman
 **/
public class SimpleDialog extends JFrame {
    public SimpleDialog( String title,
						 JComponent comp,
						 int xOff, int yOff,
						 Dimension dim,
						 boolean exitOnClose ) {
        super( title );
        final boolean doneOnClose = exitOnClose;
        addWindowListener( new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                Window win = e.getWindow();
                win.setVisible(false);
                win.dispose();
                if ( doneOnClose ) {
                    System.exit(0);
                }
            }
        } );

        JPanel panel = new JPanel();
        panel.setLayout( new BorderLayout() );
        setContentPane( panel );
        JScrollPane sp = new JScrollPane( comp );
        panel.add( sp );
        panel.setMinimumSize( dim );
        panel.setPreferredSize( dim );
        panel.setBackground( Color.white );
        pack();

        // Center frame
        Dimension screenSize =
            Toolkit.getDefaultToolkit().getScreenSize();
        Dimension size = getSize();
        screenSize.height = screenSize.height/2;
        screenSize.width = screenSize.width/2;
        size.height = size.height/2;
        size.width = size.width/2;
        int y = screenSize.height - size.height;
        int x = screenSize.width - size.width;
        setLocation(x + xOff, y + yOff);
    }
}
