/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.*;
import javax.swing.*;
import javax.swing.table.*;

import table.*;

/**
 * @version 1.0
 */
public class SimpleTable extends JTable {

    public SimpleTable() {
        super();
        _tableModel = new SortableTableModel() {
            public boolean isCellEditable(int row, int col) {
                return false;
            }  
        };
        setModel( _tableModel );
     
        setShowGrid(false);
        setBackground( Color.white );
        _renderer = new SortButtonRenderer();

        JTableHeader header = getTableHeader();
        header.addMouseListener(
            new SortHeaderListener(header,_renderer));
    }

    /**
     * Add a row to the table model without triggering an event
     * to notify the table to update itself. After finishing
     * adding all rows, fireTableDataChanged should be called
     * to notify the table.
     *
     * @param v A row to add
     * @param key An object to associate with the row
     */
    public void addRow( Vector v, Object key ) {
        _tableModel.getDataVector().addElement( v );
        _keys.addElement( key );
    }

    /**
     * Remove all rows from the model and notify the table
     */
    public void removeAllRows() {
        Vector v = _tableModel.getDataVector();
        v.removeAllElements();
        _keys.removeAllElements();
        _tableModel.fireTableDataChanged();
    }

    /**
     * Get the key value associated with a row
     *
     * @param index Index of the row
     * @return The value associated with the row
     */
    public Object getKeyAt( int index ) {
        if ( (index < 0) ||
             (index >= _keys.size()) ) {
            return null;
        }
        // The table may be sorted
       index = _tableModel.convertViewRowToModel( index );
       return _keys.elementAt( index );
    }

    /**
     * Set the column header labels and attach the sorting
     * header renderer.
     *
     * @param names Array of one label for each column
     */
    public void setColumnNames( String[] names ) {
        _tableModel.setColumnIdentifiers( names );
        setHeaderRenderer( _renderer );
    }

    /**
     * Attach the sorting header renderer to each column
     */
    public void setHeaderRenderer(
        TableCellRenderer renderer ) {
        TableColumnModel model = getColumnModel();
        for ( int i = model.getColumnCount() - 1;
              i >= 0; i-- ) {
            model.getColumn(i).setHeaderRenderer( renderer );
        }
    }

    /**
     * Set the column widths of the table.
     *
     * @param columnWidth Array of one width for each column
     */
    public void setColumnWidths( int[] columnWidth ) {
        TableColumnModel model = getColumnModel();
        for (int i = 0; i < columnWidth.length; i++) {
            model.getColumn(i).setPreferredWidth(
                columnWidth[i] );
        }
        sizeColumnsToFit( -1 );
    }

    /**
     * Notify the table that the data has changed
     */
    public void fireTableDataChanged() {
        _tableModel.fireTableDataChanged();
    }

    /**
     * Notify the table that all the data has changed
     */
    public void fireTableStructureChanged() {
        _tableModel.fireTableStructureChanged();
        _tableModel.getIndexes();
    }

    private SortableTableModel _tableModel;
    private SortButtonRenderer _renderer;
    private Vector _keys = new Vector();
}
