/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import netscape.ldap.*;

import util.*;

/**
 * TestTree
 * 
 *
 * @version 1.0
 * @author rweltman
 **/
public class TestTree implements IDirNodeListener {

    TestTree( String[] args ) {

        // Parse arguments
        String host = "localhost";
        int port = 389;
        String authDN = null;
        String authPassword = null;
        boolean invalid = false;
        int nPanes = 1;
        int debugLevel = 0;
        boolean showLeaf = false;
        boolean showPrivate = false;
        boolean followReferrals = false;
        String baseDN = null;
        int i = 0;
        while ( i < args.length ) {
            if (args[i].compareTo("-D" ) == 0 ) {
                if ( (i + 1) < args.length ) {
                    authDN = args[i+1];
                    i++;
                }
            } else if ( args[i].compareTo("-w" ) == 0 ) {
                if ( (i + 1) < args.length ) {
                    authPassword = args[i+1];
                    i++;
                }
            } else if ( args[i].compareTo("-h" ) == 0 ) {
                if ( (i + 1) < args.length ) {
                    host = args[i+1];
                    i++;
                }
            } else if ( args[i].compareTo("-b" ) == 0 ) {
                if ( (i + 1) < args.length ) {
                    baseDN = args[i+1];
                    i++;
                }
            } else if ( args[i].compareTo("-p" ) == 0 ) {
                if ( (i + 1) < args.length ) {
                    port = Integer.parseInt(args[i+1]);
                    i++;
                }
            } else if ( args[i].compareTo("-debug" ) == 0 ) {
                if ( (i + 1) < args.length ) {
                    debugLevel = Integer.parseInt(args[i+1]);
                    i++;
                }
            } else if ( args[i].compareTo("-panes" ) == 0 ) {
                if ( (i + 1) < args.length ) {
                    nPanes = Integer.parseInt(args[i+1]);
                    invalid = ( (nPanes < 1) || (nPanes > 3) );
                    i++;
                }
            } else if ( args[i].compareTo("-leaf" ) == 0 ) {
                showLeaf = true;
            } else if ( args[i].compareTo("-private" ) == 0 ) {
                showPrivate = true;
            } else if ( args[i].compareTo("-referrals" ) == 0 ) {
                followReferrals = true;
            } else {
                invalid = true;
                break;
            }
            i++;
        }

        if ( invalid ) {
            System.err.println( "Usage: TestTree [-h HOST] " +
                                "[-p PORT] [-D AUTHDN] "+
                                "[-w AUTHPASSWORD] " +
                                "[-b BASEDN] [-panes {1,2,3}] " +
                                "[-leaf] " +
                                "[-private] [-referrals] " +
                                "[-debug DEBUGLEVEL]" );
            System.exit( 1 );
        }
        Debug.setTraceLevel( debugLevel );

        JFrame frame = new JFrame("TestTree");
        frame.addWindowListener( new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                Window win = e.getWindow();
                win.setVisible(false);
                win.dispose();
                System.exit(0);
            }
        } );

        try {
            LDAPConnection ldc =
                DirUtil.getLDAPConnection( host,
                                           port,
                                           authDN,
                                           authPassword );
            DirModel model = new DirModel( ldc );
            model.setShowsPrivateSuffixes( showPrivate );
            model.setAllowsLeafNodes( showLeaf );
            model.setReferralsEnabled( followReferrals );
            if ( baseDN != null ) {
                model.initialize( new DirNode( model, baseDN ) );
            } else {
                model.initialize( null );
            }

            // Create a JTree for displaying directory data
            DirTree tree = new DirTree( model );
            tree.addDirNodeListener( this );
            JScrollPane treeSp = new JScrollPane( tree );
            treeSp.setBorder(
                new BevelBorder(
                    BevelBorder.LOWERED,
                    UIManager.getColor("controlHighlight"),
                    UIManager.getColor("control"),
                    UIManager.getColor("controlDkShadow"),
                    UIManager.getColor("controlShadow")));

            if ( nPanes > 1 ) {
                // Show both a tree and a table
                final SimpleTable table = new SimpleTable();
                // Hook the table to the tree with an adapter
                EntryListAdapter lister =
                    new EntryListAdapter( ldc,
                                          table );
                tree.addDirNodeListener( lister );
                // A panel is required to keep the background
                // the right color
                JPanel tablePanel =
                    addTableToScrollPanel( table );

                JSplitPane splitPane =
                    new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
                                   treeSp, tablePanel);
                splitPane.setBorder(
                    new EmptyBorder(6, 6, 6, 6) );

                if ( nPanes > 2 ) {
                    // Add a property table to show all
                    // properties of a selected table row
                    final PropertyTable ptable =
                        new PropertyTable( ldc, "" );
                    tablePanel =
                        addTableToScrollPanel( ptable );

                    // A vertical splitter pane, with the
                    // property table at the bottom
                    JSplitPane triPane =
                        new JSplitPane(
                            JSplitPane.VERTICAL_SPLIT,
                            splitPane, tablePanel);
                    triPane.setBorder(
                        new EmptyBorder(6, 6, 6, 6) );
                    triPane.setDividerLocation( 200 );
                    frame.getContentPane().add( triPane );
                    frame.setSize( new Dimension( 650, 450 ) );
                    // Listen for selection changes in the
                    // simple table and notify the property
                    // table
                    ListSelectionModel m =
                        table.getSelectionModel();
                    m.addListSelectionListener(
                        new ListSelectionListener() {
                            public void valueChanged(
                                ListSelectionEvent e) {
                                int index =
                                    table.getSelectedRow();
                                String dn =(String)
                                    table.getKeyAt( index );
                                ptable.setDN( dn );
                            }
                    });
                } else {
                    // Two panes
                    frame.getContentPane().add( splitPane );
                    frame.setSize( new Dimension( 650, 300 ) );
                }
            } else {
                // One pane - just show the tree
                frame.getContentPane().add( treeSp );
                frame.setSize( new Dimension( 200, 300 ) );
            }

            // Center frame
            Dimension screenSize =
                Toolkit.getDefaultToolkit().getScreenSize();
            Dimension size = frame.getSize();
            screenSize.height = screenSize.height/2;
            screenSize.width = screenSize.width/2;
            size.height = size.height/2;
            size.width = size.width/2;
            int y = screenSize.height - size.height;
            int x = screenSize.width - size.width;
            frame.setLocation(x, y);
            frame.show();
        } catch ( Exception e ) {
            System.err.println( e );
            System.exit( 1 );
        }
    }
 
    protected JPanel addTableToScrollPanel( JTable table ) {
        JPanel tablePanel = new JPanel();
        tablePanel.setLayout( new BorderLayout() );
        tablePanel.setBackground( table.getBackground() );
        JScrollPane tableSp = new JScrollPane( table );
        tableSp.setBorder(
            new BevelBorder(
                BevelBorder.LOWERED,
                UIManager.getColor("controlHighlight"),
                UIManager.getColor("control"),
                UIManager.getColor("controlDkShadow"),
                UIManager.getColor("controlShadow")));
        tablePanel.add( tableSp );
        return tablePanel;
    }

    /**
     * The selection changed.
     *
     * @param nodes Array of selected tree nodes
     */
    public void selectionChanged( IDirNode[] nodes ) {
        // We don't do anything with the event here; it is
        // handled by the adapter
        System.out.println( "TestTree.selectionChanged: " +
                            nodes[0] );
    }

    /**
     * An action was invoked using the mouse or keyboard.
     *
     * @param ev Object indicating the type of event.
     */
    public void actionInvoked( DirNodeEvent ev ) {
        System.out.println( "TestTree.actionInvoked: " +
                            ev.getNodes()[0] );
    }

    public static void main(String[] args) {
        new TestTree( args );
    }
}
