import netscape.ldap.*;

public class TestMultiple {
    LDAPConnection ld = new LDAPConnection(); // shared among threads
    TestMultiple( String[] args ) {
        if ( args.length < 4 ) {
            System.err.println( "Usage: TestMultiple <host> <port> " +
                                "<authdn> <password>" );
            System.exit( 1 );
        }
        try {
            // Connect and authenticate on behalf of all tasks
            ld.connect( args[0], Integer.parseInt(args[1]) );
            ld.authenticate( args[2], args[3] );
        } catch( LDAPException e ) {
            System.out.println( "Error: " + e.toString() );
            System.exit( 1 );
        }
        // Create and start a task to process all person entries under
        // "ou=People,o=Airius.com"
        DoASearch search = new DoASearch( "ou=People,o=Airius.com",
                                          "objectclass=person" );
        Thread th1 = new Thread(search, "searchPeople");
        th1.start();
        // Create and start a task to change Babs' common name
        ChangeName change =
            new ChangeName( "uid=bjensen,ou=People,o=Airius.com",
                            "Babs B. Jensen" );
        Thread th2 = new Thread(change, "changeBabs");
        th2.start();
        // At this point, both tasks are running in parallel
        // Wait for both to finish, and then exit
        try {
            th1.join();
            th2.join();
        } catch ( InterruptedException e ) {
        }
        System.exit( 0 );
    }

    public static void main( String[] args ) {
        new TestMultiple( args );
    }

    // A command class - instantiate it in a thread to do a search
    class DoASearch implements Runnable {
        String base, filter;
        DoASearch( String base, String filter ) {
            this.base = base;
            this.filter = filter;
        }
        public void run() {
            LDAPSearchResults res;
            try {
                // Use the LDAPConnection object of the outer class
                res = ld.search( base,
                                 ld.SCOPE_SUB,
                                 filter,
                                 null,
                                 false );
            } catch ( LDAPException e ) {
                System.out.println( "Error: " + e.toString() );
                return;
            }
            while ( res.hasMoreElements() ) {
                try {
                    LDAPEntry entry = res.next(); // next directory entry
                    System.out.println( entry.getDN() );
                    // Do something with the entry...
                } catch ( LDAPException ex ) {
                    System.out.println( "Error: " + ex.toString() );
                    continue;
                }
            }
        }
    }

    // A command class - instantiate it in a thread to change the
    // common name of an entry
    class ChangeName implements Runnable {
        String dn, newName;
        ChangeName( String dn, String newName ) {
            this.dn = dn;
            this.newName = newName;
        }
        public void run() {
            try {
                // Use the LDAPConnection object of the outer class
                ld.modify( dn, new LDAPModification(
                                   LDAPModification.REPLACE,
                                   new LDAPAttribute( "cn", newName ) ) );
                System.out.println( "Changed name of " + dn + ": "
                                    + newName );
            } catch ( LDAPException e ) {
                System.out.println( "Error: " + e.toString() );
            }
        }
    }
}
