/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */
import netscape.ldap.*;
import java.util.*;

/**
 * Demonstrate asynchronous authentication and search
 * 
 *
 * @version 1.0
 * @author rweltman
 **/
public class AsynchSearch {
    public static void main( String[] args ) {
        String MY_FILTER = "objectclass=*";
        String MY_SEARCHBASE = "o=Airius.com";
        LDAPConnection ld = new LDAPConnection();
        try {
            ld.connect( "localhost", 389 );
            // Asynchronous authentication
            LDAPResponseListener r =
                ld.bind( "uid=jvedder, ou=People, o=airius.com",
                         "befitting",
                         (LDAPResponseListener)null );
            // Do something else, just to show that we're not
            // blocked yet
            System.out.println( "Started authenticating" );
            // Wait until it completes
            LDAPResponse response = r.getResponse();
            // Did the authentication succeed?
            int resultCode = response.getResultCode();
            if (resultCode != LDAPException.SUCCESS) {
                // Do what the synchronous interface does - throw
                // an exception
                String err =
                    LDAPException.errorCodeToString( resultCode );
                throw new LDAPException ( err,
                                          resultCode,
                                          response.getErrorMessage(),
                                          response.getMatchedDN());
            }

            // Start searching. Pass null for LDAPSearchListener to
            // obtain a new one.
            LDAPSearchListener l =
                ld.search( MY_SEARCHBASE,
                           ld.SCOPE_ONE,
                           MY_FILTER,
                           null,
                           false,
                           (LDAPSearchListener)null );
            // Loop on results until finished
            LDAPMessage msg;
            while( (msg = l.getResponse()) != null ) {
                // Next directory entry
                if ( msg instanceof LDAPSearchResultReference ) {
                    String[] urls =
                        ((LDAPSearchResultReference)msg).getUrls();
                    // Do something with the referrals...
                } else if ( msg instanceof LDAPSearchResult ) {
                    LDAPEntry entry =
                        ((LDAPSearchResult)msg).getEntry();
                    // The rest of the processing is the same as for
                    // a synchronous search
                    System.out.println( entry.getDN() );
                } else {
                    // A search response
                    LDAPResponse res = (LDAPResponse)msg;
                    int status = res.getResultCode();
                    if ( status == LDAPException.SUCCESS ) {
                        // Nothing to do
                    } else {
                        String err =
                            LDAPException.errorCodeToString( status );
                        throw new LDAPException( err,
                                                 status,
                                                 res.getErrorMessage(),
                                                 res.getMatchedDN() );
                    }
                }
            }
        } catch( LDAPException e ) {
            System.out.println( "Error: " + e.toString() );
        }

        // Done, so disconnect
        if ( (ld != null) && ld.isConnected() ) {
            try {
                ld.disconnect();
            } catch ( LDAPException e ) {
                System.out.println( e.toString() );
            }
        }
        System.exit( 0 );
    }
}
