/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset:4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */


import netscape.ldap.*;
import java.util.*;

/**
 * Delete a branch point and all records under it
 *<P>
 * USE THIS PROGRAM WITH CARE. IT WILL ERASE YOUR ENTIRE
 * DIRECTORY IF YOU SET A START POINT OF THE ROOT AND HAVE
 * SUFFICIENT ACCESS RIGHTS.<BR>
 * !!!! YOU HAVE BEEN WARNED !!!!
 */
public class BranchDelete {

    /**
     * Delete an entry or subtree
     *
     * @param args host, port, dn, authdn, password, YES|N
     */
    public static void main( String[] args ) {

        if ( args.length != 6 ) {
            System.out.println( "Usage: java BranchDelete " +
                                "<host> <port> " +
                                "<dn> <authdn> <password> " +
                                "YES|N" );
            System.out.println( "Example:" );
            System.out.println( "    java BranchDelete " +
                                "localhost 389 " +
                                "\"ou=notneeded," +
                                "o=airius.com\" " +
                                "\"cn=Directory Manager\" " +
                                "password YES" );
            System.out.println( "YES|N is a confirmation " +
                                "for actually doing the " +
                                "deletion");
            System.out.println( "YES must be in all caps " +
                                "and spelled out!" );
            System.exit(1);
        }

        String host = args[0];
        int port = Integer.parseInt( args[1] );
        String branch = args[2];
        String authid = args[3];
        String authpw = args[4];

        boolean doDelete = false;
        if ( args[5].equals( "YES" ) ) {
            System.out.println( "Deleting " + branch + '!' );
            doDelete = true;
        } else {
            System.out.println( "Not deleting");
        }

        LDAPConnection ld = new LDAPConnection();
        try {
            // Connect to server and authenticate
            ld.connect( host, port, authid, authpw );
            // Start deleting from the selected point
            delete( branch, ld, doDelete );
        } catch( LDAPException e ) {
            System.out.println( e.toString() );
        }

        // Done, so disconnect
        if ( (ld != null) && ld.isConnected() ) {
            try {
                ld.disconnect();
            } catch ( LDAPException e ) {
                System.out.println( e.toString() );
            }
        }
        System.exit(0);
    }

    /**
     * Delete an entry, recursing if the entry has children
     *
     * @param dn DN of the entry to delete
     * @param ld Active connection to server
     * @param doDelete <CODE>true</CODE> if the entries really
     * are to be deleted
     */
    public static void delete( String dn,
                               LDAPConnection ld,
                               boolean doDelete ) {
        String theDN = "";

        try {
            LDAPSearchConstraints cons =
                ld.getSearchConstraints();
            // Retrieve all results at once
            cons.setBatchSize( 0 );
            cons.setMaxResults( 0 );
            // Find all immediate child nodes, return no
            // attributes
            LDAPSearchResults res =
                ld.search( dn,
                           ld.SCOPE_ONE,
                           "objectclass=*",
                           new String[] {LDAPv3.NO_ATTRS},
                           false,
                           cons );
            // Recurse on entries under this entry
            while ( res.hasMoreElements() ) {
                try {
                    // Next directory entry
                    LDAPEntry entry = res.next();
                    theDN = entry.getDN();
                    // Recurse down
                    delete( theDN, ld, doDelete );
                } catch ( LDAPReferralException e ) {
                    // Do not follow referrals, just list them
                    System.out.println( "Search reference: " );
                    LDAPUrl refUrls[] = e.getURLs();
                    for ( int i = 0; i < refUrls.length; i++ ) {
                       System.out.println(
                           "  " + refUrls[i].getUrl() );
                    }
                    continue;
                } catch ( LDAPException e ) {
                    System.out.println( e.toString() );
                    continue;
                }
            }

            // At this point, the DN represents a leaf node,
            // so stop recursing and delete the node
            try {
                if ( doDelete ) {
                    ld.delete( dn );
                    System.out.println( dn + " deleted" );
                }
                else {
                    System.out.println( dn +
                                        " would be deleted" );
                }
            } catch (LDAPException e) {
                System.out.println( e.toString() );
            } 

        } catch( LDAPException e ) {
            System.out.println( e.toString() );
        }

        return;
    }
}
