/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset:4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.filechooser.FileFilter;
import java.net.*;
import netscape.ldap.*;

/**
 * A dialog for viewing and updating the jpegPhoto attribute
 * of an entry in a directory
 * 
 * @version 1.0
 */
public class JPEGLDAP extends JFrame {
   
   /**
    * Launch a dialog
    * 
    * @param args host and port of LDAP server
    */
   public static void main(String args[]) {
       try {
           if ( args.length != 2 ) {
               System.out.println( "Usage: java JPEGLDAP " +
                                   "<host> <port>" );
               System.exit( 1 );
           }
           String host = args[0];
           int port = Integer.parseInt( args[1] );

           // Enable the following code if you want the Look
           // and Feel to be set to the Look and Feel of the
           // native system.
           /*
           try {
                UIManager.setLookAndFeel(
                    UIManager.getSystemLookAndFeelClassName());
           } catch (Exception e) { 
           }
           */

           // Create a new instance of the application frame
           // and make it visible.
           JFrame frame = new JPEGLDAP( host, port );
           frame.pack();
           frame.setVisible(true);
       } catch (Exception e) {
           e.printStackTrace();
           // Ensure the application exits with an error
           // condition.
           System.exit(1);
       }
   }

    /**
     * Constructor
     * 
     * @param host Host name of LDAP server
     * @param port Port number of LDAP server
     */
    public JPEGLDAP( String host, int port ) {
        super( "LDAP JPEG Photo Handler" );

        setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
        setVisible(false);

        DNEntryField = new JTextField();
        DNEntryField.setPreferredSize(
            new Dimension( 300, 24 ) );
        DNEntryField.setMaximumSize(
            new Dimension( 300, 24 ) );
        JLabel DNLabel = new JLabel( "DN: " );
        DNLabel.setHorizontalAlignment(SwingConstants.RIGHT);
        DNLabel.setLabelFor(DNEntryField);

        JLabel pswdFieldLbl = new JLabel( "Password: " );
        pswdFieldLbl.setHorizontalAlignment(
            SwingConstants.RIGHT);
        pswdFieldLbl.setLabelFor(passwordField);
        passwordField = new JPasswordField();
        passwordField.setPreferredSize(
            new Dimension( 300, 24 ) );
        passwordField.setMaximumSize(
            new Dimension( 300, 24 ) );

        // Read the default images into memory for display
        noLDAP = getFileData("noldap.jpg");
        noFile = getFileData("nofile.jpg");
        if ( (noLDAP == null) || (noFile == null) ) {
            System.out.println("Unable to read default files");
            System.exit(1);
        }

        // JPEG image data for LDAP image
        currentPhotoIcon =
            adjustImageSize( new ImageIcon(noLDAP), dim );
        currentPhoto = new JLabel( currentPhotoIcon );
        currentPhotoIcon.setImageObserver(currentPhoto);
        currentPhoto.setPreferredSize( dim );
        currentPhoto.setMaximumSize( dim );

        // JPEG image data for file image
        newPhotoIcon =
            adjustImageSize( new ImageIcon(noFile), dim );
        newPhoto = new JLabel( newPhotoIcon );
        newPhotoIcon.setImageObserver(newPhoto);
        newPhoto.setPreferredSize( dim );
        newPhoto.setMaximumSize( dim );

        Dimension buttonSize = new Dimension( 110, 16 );
        saveNewBtn = new JButton( "< Save" );
        saveNewBtn.setPreferredSize( buttonSize );
        saveNewBtn.setMinimumSize( buttonSize );
        saveNewBtn.addActionListener(new saveNewJPEG());
        // Nothing to save yet
        saveNewBtn.setEnabled(false);

        JLabel currentPhotoLbl =
            new JLabel( "Current Photo" );
        currentPhotoLbl.setHorizontalAlignment(
            SwingConstants.CENTER);
        currentPhotoLbl.setLabelFor(currentPhoto);

        JLabel newPhotoLbl = new JLabel( "New Photo" );
        newPhotoLbl.setHorizontalAlignment(
            SwingConstants.CENTER);
        newPhotoLbl.setLabelFor(newPhoto);

        loadNewBtn = new JButton( "Select Photo" );
        loadNewBtn.setPreferredSize( buttonSize );
        loadNewBtn.setMinimumSize( buttonSize );
        loadNewBtn.addActionListener(new GetNewJPEG());
        loadNewBtn.setEnabled(false);  // Not valid yet

        JButton retrieveBtn = new JButton( "Get Entry" );
        retrieveBtn.setPreferredSize( buttonSize );
        retrieveBtn.setMinimumSize( buttonSize );
        retrieveBtn.addActionListener(new GetLDAPEntry());

        // Now lay out all the components
        Box pane = new Box( BoxLayout.Y_AXIS );
        getContentPane().add( pane );

        Box textBox = new Box( BoxLayout.X_AXIS );
        Box DNBox = new Box( BoxLayout.X_AXIS );
        DNBox.add( Box.createGlue() );
        DNBox.add( DNLabel );
        DNBox.add( DNEntryField );
        Box passwordBox = new Box( BoxLayout.X_AXIS );
        passwordBox.add( Box.createGlue() );
        passwordBox.add( pswdFieldLbl );
        passwordBox.add( passwordField );
        Box inputBox = new Box( BoxLayout.Y_AXIS );
        inputBox.add( DNBox );
        inputBox.add( Box.createVerticalStrut( 3 ) );
        inputBox.add( passwordBox );

        textBox.add( inputBox );
        textBox.add( Box.createHorizontalStrut( 6 ) );
        textBox.add( retrieveBtn );

        pane.add( Box.createGlue() );
        pane.add( textBox );
        pane.add( Box.createVerticalStrut( 6 ) );
        pane.add( Box.createGlue() );

        Box theRestBox = new Box( BoxLayout.X_AXIS );
        pane.add( theRestBox );

        Box imageBox = new Box( BoxLayout.X_AXIS );
        Box currentBox = new Box( BoxLayout.Y_AXIS );
        currentBox.add( currentPhotoLbl );
        currentBox.add( currentPhoto );
        imageBox.add( currentBox );
        imageBox.add( Box.createHorizontalStrut( 6 ) );
        imageBox.add( saveNewBtn );
        imageBox.add( Box.createHorizontalStrut( 6 ) );
        Box newBox = new Box( BoxLayout.Y_AXIS );
        newBox.add( newPhotoLbl );
        newBox.add( newPhoto );
        imageBox.add( newBox );

        theRestBox.add( Box.createGlue() );
        theRestBox.add( imageBox );
        theRestBox.add( Box.createHorizontalStrut( 6 ) );
        theRestBox.add( Box.createGlue() );
        theRestBox.add( loadNewBtn );

        // Handle window closing action
        addWindowListener(new WindowCloser());

        // Connect to server
        try {
            ld = new LDAPConnection();
            ld.connect( host, port );
        } catch (LDAPException e) {
            System.out.println("Connect error: "+ e.toString());
            System.exit(1);
        }
        System.out.println( "Connected to LDAP server " +
                            host );
    }

    /**
     * Read the contents of a file into memory
     * 
     * @param fname Name of the file
     * @return The contents of the file
     */
    public byte[] getFileData( String fname ) {
        byte[] data = null;
        RandomAccessFile rf = null;
        try {
            rf = new RandomAccessFile(fname,"r");
            data = new byte[(int)rf.length()];
            rf.readFully(data);
        } catch (IOException e) {
            e.printStackTrace();
        }
        try {
            if (rf != null) {
                rf.close();
            }
        } catch (IOException e) { }
        return data;
    }

    /**
     * If the image is larger than the specified max, then scale it down,
     * otherwise return the input image
     *
     * @param icon Input image
     * @param dim Max width and height
     * @return Scaled-down image, or original image
     */
    private ImageIcon adjustImageSize( ImageIcon icon, Dimension dim ) {
        if ( (icon.getIconHeight() > dim.height) ||
             (icon.getIconWidth() > dim.width) ) {
            Image img = icon.getImage();
            float hRatio = (float)icon.getIconHeight() / dim.height;
            float wRatio = (float)icon.getIconWidth() / dim.width;
            int h, w;
            if ( hRatio > wRatio ) {
                h = dim.height;
                w = (int)(icon.getIconWidth() / hRatio);
            } else {
                w = dim.width;
                h = (int)(icon.getIconHeight() / wRatio);
            }
            System.out.println( "Scaling from (" +
                                icon.getIconWidth() + ',' +
                                icon.getIconHeight() + ") to (" +
                                w + ',' + h + ')' );
            img = img.getScaledInstance( w, h, img.SCALE_SMOOTH );
            return new ImageIcon( img );
        } else {
            System.out.println( "Image is (" +
                                icon.getIconWidth() + ',' +
                                icon.getIconHeight() + ')' );
            return icon;
        }
    }

    /**
     * Handler for New button
     */
    class GetNewJPEG implements ActionListener {
        /**
         * Called when a button is pressed
         * 
         * @param e Event from New button
         */
        public void actionPerformed(ActionEvent e) {
            // Select a file
            JFileChooser chooser = new JFileChooser(directory);
            // Make sure it is a JPEG file
            JPEGFileFilter jpgfilter = new JPEGFileFilter(); 
            chooser.setFileFilter(jpgfilter);
            int returnVal =
                chooser.showOpenDialog(JPEGLDAP.this);
            directory =
                chooser.getCurrentDirectory().getPath();
            if ( returnVal == JFileChooser.APPROVE_OPTION ) {  
                File f = chooser.getSelectedFile();
                String fullname = f.getPath(); 
                // Load the JPEG data
                newFilePhoto = getFileData( fullname );
                if ( newFilePhoto == null ) {
                    System.out.println( "Unable to read " +
                                        "file " +
                                        fullname );
                    return;
                }
                // Create image object
                newPhotoIcon =
                    adjustImageSize(
                        new ImageIcon(newFilePhoto), dim );
                newPhoto.setIcon(newPhotoIcon);
                newPhoto.setPreferredSize( dim );
                newPhoto.setMaximumSize( dim );
                // There is now an image to save
                saveNewBtn.setEnabled( true );
                return;
            }
        }
    }

    /**
     * Handler for Get button
     */
    class GetLDAPEntry implements ActionListener {
        /**
         * Called when a button is pressed
         * 
         * @param e Event from Get button
         */
        public void actionPerformed(ActionEvent e) {

            byte[] theLDAPPhoto = null;
            DNRead = DNEntryField.getText();
            DNPswd =
                new String(passwordField.getPassword());
            if ( (DNRead.length() == 0) ||
                 (DNPswd.length() == 0) ) {
                System.out.println("DN and Password must " +
                                   "be entered!");
                return;
            }
            // Go get the data
            try {
                // Authenticate with the credentials of the
                // owner of the photo
                ld.authenticate(DNRead,DNPswd);
                // Read the entry
                LDAPEntry entry = ld.read( DNRead, ATTRS );
                // Extract the jpegPhoto attribute
                LDAPAttribute attr =
                    entry.getAttribute(ATTRS[0]);
                if ( attr != null ) {
                    // Get the values as byte arrays
                    Enumeration enumVals =
                        attr.getByteValues();
                    // Get the first value - if there's more
                    // than one
                    if ( enumVals.hasMoreElements() ) {
                        theLDAPPhoto =
                            (byte[])enumVals.nextElement();
                    }
                } else {
                    System.out.println( "No photo in the " +
                                        "directory for " +
                                        DNRead );
                }
                if ( (attr == null) ||
                     (theLDAPPhoto == null) ) {
                    // Reset images to mark this condition
                    currentPhotoIcon =
                        adjustImageSize( new ImageIcon(noLDAP), dim );
                    currentPhoto.setIcon(currentPhotoIcon);
                    newPhotoIcon =
                        adjustImageSize( new ImageIcon(noFile), dim );
                    newPhoto.setIcon(newPhotoIcon);
                    newFilePhoto = null;
                    // Nothing to save now
                    saveNewBtn.setEnabled(false);
                    loadNewBtn.setEnabled(true); 
                } else {
                    // Show image from LDAP and clear right
                    // image
                    newFilePhoto = theLDAPPhoto;
                    currentPhotoIcon =
                        adjustImageSize(
                            new ImageIcon(newFilePhoto), dim );
                    currentPhoto.setIcon(currentPhotoIcon);
                    newPhotoIcon =
                        adjustImageSize( new ImageIcon(noFile), dim );
                    newPhoto.setIcon(newPhotoIcon);
                    loadNewBtn.setEnabled(true);
                }
                currentPhoto.setPreferredSize( dim );
                currentPhoto.setMaximumSize( dim );
                newPhoto.setPreferredSize( dim );
                newPhoto.setMaximumSize( dim );
 
            } catch ( LDAPException le ) {
                System.out.println( le.toString() ); 
            }
            return;
        }
    }
    

    /**
     * Handler for Save button
     */
    class saveNewJPEG implements ActionListener {
      /**
       * Called when a button is pressed
       * 
       * @param e Event from Save button
       */
        public void actionPerformed(ActionEvent e) {

            // Save the image in the directory
            try {
                // Authenticate with the credentials of the
                // owner of the photo
                ld.authenticate(DNRead,DNPswd);
                // Write image to directory
                LDAPAttribute attrphoto = 
                    new LDAPAttribute(ATTRS[0],newFilePhoto);
                LDAPModification mod =
                    new LDAPModification(
                        LDAPModification.REPLACE,
                        attrphoto);
                ld.modify( DNRead,mod );

                // Show current (LDAP) image (same as right)
                currentPhotoIcon =
                    adjustImageSize(
                        new ImageIcon(newFilePhoto), dim );
                currentPhoto.setIcon(currentPhotoIcon);
                currentPhoto.setPreferredSize( dim );
                currentPhoto.setMaximumSize( dim );
                System.out.println("Photo saved to DN: " + DNRead);
            
            } catch ( LDAPException le ) {
                System.out.println( le.toString() ); 
            }
            return;
        }
    }
    
    /**
     * Handler for window closing events
     */
    class WindowCloser extends WindowAdapter {
        /** Close the window and application
         * 
         * @param event Event for window closing
         */
        public void windowClosing(WindowEvent event) {
            Window win = event.getWindow();
            win.setVisible(false);
            win.dispose();
            // Close down LDAP connection
            if ( (ld != null) && (ld.isConnected()) ) {
                try {
                    ld.disconnect();
                } catch (LDAPException e) {
                    System.out.println( e.toString() );
                }
            }
            System.exit(0);
        }
    }

    /**
     * Helper class for JFileChooser to force selection of a
     * JPEG file
     */
    class JPEGFileFilter extends FileFilter {
        /**
         * Validate a file name
         * 
         * @param f A file name
         * @return <CODE>true</CODE> if it is a valid JPEG file name
         */
        public boolean accept( File f ) {
            if ( f == null ) {
                return false;
            }
            if ( f.isDirectory() ) {
                return true;
            }
            String name = (f.getName()).toLowerCase();
            return ( (name.endsWith("jpg")) ||
                     (name.endsWith("jpeg")) );
        }

        /**
         * Returns a description string for the file filter
         * 
         * @return Description field for file filter
         */
        public String getDescription() {
            return "JPEG Image Files (*.jpg, *.jpeg)";
        }
    }

    private LDAPConnection ld = null;
    static final private String[] ATTRS = {"jpegPhoto"};
    // Size of image
    static final private Dimension dim = new Dimension( 144, 156 );
    // Stores image of jpeg for no ldap photo
    private byte[] noLDAP;
    // Stores image of jpeg for no file photo
    private byte[] noFile;
    // When user selects new file to send
    private byte[] newFilePhoto;
    // Track what we read for current display
    private String DNRead = "";
    private String DNPswd = "";
    // Get the working directory
    private String directory =
        System.getProperty("user.dir");
    private JTextField DNEntryField;
    private JPasswordField passwordField;
    private ImageIcon currentPhotoIcon; 
    private JLabel currentPhoto;
    private JLabel newPhoto;
    private ImageIcon newPhotoIcon; 
    private JButton saveNewBtn;
    private JButton loadNewBtn;
}
