/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public License
 * Version 1.0 (the "NPL"); you may not use this file except in
 * compliance with the NPL.  You may obtain a copy of the NPL at
 * http://www.mozilla.org/NPL/
 *
 * Software distributed under the NPL is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the NPL
 * for the specific language governing rights and limitations under the
 * NPL.
 *
 * The Initial Developer of this code under the NPL is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1999 Netscape Communications Corporation.  All Rights
 * Reserved.
 */
import java.io.*;
import java.net.*;
import java.util.*;
import netscape.ldap.*;

/**
 * JSLDAPConnection.java<P>
 * JAVA class for Java Script to access LDAP server, which
 * wraps the Netscape Directory SDK for Java LDAPConnection class
 * to trap exceptions and provide some additional aid for Java/JS
 * transitions. Doesn't support SSL or authentication other than
 * simple auth.
 */

public class JSLDAPConnection extends LDAPConnection {
	public JSLDAPConnection() {
		super();
	}

	/**
	 * Connect to LDAP server
	 *
	 * @return 0 on success, an LDAP error code failure
	 */
	public int safeConnect( int version, String host, int port,
							String authdn, String authpw ) {
		try {
			connect( version, host, port, authdn, authpw );
		} catch( LDAPException e ) {
			return (_errorCode = e.getLDAPResultCode());
		}
        _errorCode = 0;
		return 0;
    }


	/**
	 * Disconnect from LDAP server
	 *
	 * @return 0 on success, an LDAP error code failure
	 */
	public int safeDisconnect() {
		try {
			disconnect();
		} catch( LDAPException e ){
			return (_errorCode = e.getLDAPResultCode());
		}
        _errorCode = 0;
		return 0;
    }

    /**
     * Performs the search specified by the criteria that you enter.
     * This method also allows you to specify constraints for the search
     * (such as the maximum number of entries to find or the
     * maximum time to wait for search results). <P>
     *
     * @param base The base distinguished name to search from
     * @param scope The scope of the entries to search.
     * @param filter Search filter specifying the search criteria.
     * @param attrs List of attributes that you want returned in the search
     * results.
     * @param cons Constraints specific to this search (for example, the
     * maximum number of entries to return).
     * @param attrsOnly If true, returns the names but not the values of the
     * attributes found.  If false, returns the names and  values for
     * attributes found
     * @return JSLDAPSearchResults The results of the search,
	 * or <CODE>null</CODE> on failure. If <CODE>null</CODE>, call
	 * getErrorCode() to get the LDAP error code.
     */
    public JSLDAPSearchResults
        safeSearch( String base,
                    int scope,
                    String filter,
                    String[] attrs,
                    boolean attrsOnly,
                    LDAPSearchConstraints cons ) {

		try {
			LDAPSearchResults res = search( base, scope,
                                            filter, attrs,
                                            attrsOnly, cons );
            _errorCode = 0;
            return new JSLDAPSearchResults( res );
		} catch( LDAPException e ) {
			_errorCode = e.getLDAPResultCode();
            return null;
		}
	}

    /**
     * Reads the entry for the specified distiguished name (DN)
     * and retrieves all attributes for the entry.
     *
     * @param DN Distinguished name of the entry that you want
     * to retrieve.
     * @return The specified entry or null if the entry is not
     * found.
     */
    public LDAPEntry safeRead ( String DN ) {
		try {
			LDAPEntry entry = read( DN );
            _errorCode = 0;
            return entry;
		} catch( LDAPException e ) {
			_errorCode = e.getLDAPResultCode();
            return null;
		}
    }

	/**
	 * Get latest LDAP error code
	 *
	 * @return Latest LDAP error code
	 */
	public int getErrorCode() {
		return _errorCode;
	}

    private int _errorCode = 0;
}
