/*
 * @(#) Cache.java
 *
 * Copyright 1997 Netscape Communications Corp.  All rights reserved.
 */
package netscape.ldap.tests;

import netscape.ldap.*;
import netscape.ldap.controls.*;
import java.util.*;
import java.io.*;

/**
 * This test program is to test the performance of the memory cache. It takes two arguments
 * which are the configuration file (config.txt as an example) and the data file (data-c.txt
 * as an example).
 */
public class Cache {
  public static final int INTERVAL = 999;

  public static void main(String[] argv) {

    FileReader configFile = null;
    FileReader dataFile = null;
    String hostname = "localhost";
    int port = 389;
    String bindDN = "";
    String passwd = "";

    try {
      // read the configuration file: config.txt
      configFile = new FileReader(argv[0]);

      // read the data file: data-c.txt which is generated by randomData.java
      dataFile = new FileReader(argv[1]);
    } catch (FileNotFoundException e) {
      System.out.println("Exception: "+e);
    }

    BufferedReader in1 = new BufferedReader(configFile);
    try {
      hostname = in1.readLine();
      port = Integer.parseInt(in1.readLine());
      bindDN = in1.readLine();
      passwd = in1.readLine();
    } catch (IOException e) {
      System.out.println("Exception: "+e);
    }

    BufferedReader in2 = new BufferedReader(dataFile);
    LDAPConnection ld = null;
    LDAPSearchConstraints cons = null;
    LDAPCache cache = new LDAPCache(300, 200000);

    try {
      ld = new LDAPConnection();
      ld.setCache(cache);
      ld.connect(hostname, port);
      ld.authenticate(3, bindDN, passwd);
    } catch (LDAPException e) {
      System.out.println("Exception: " +e);
    }

    cons = ld.getSearchConstraints(); 
    cons.setBatchSize(1);

    int searchCounter = 0;
    Date date = new Date();
    long currTime = date.getTime();
    System.out.println("***** Start Search -> " + currTime);
    while (true) {
      Vector input1 = new Vector();
      String[] attrs = null;
      boolean finish = false;
      String str = null;
      int count = 0;
      while (true) {
        try {
          str = in2.readLine();
        } catch (IOException ie) {
          System.out.println("Exception: "+ie);
        }
        if ((str == null) || (str.length() == 0))
          break;
        else if (count == 3) {
          StringTokenizer t = new StringTokenizer(str);
          attrs = new String[t.countTokens()];
          int j=0;
          while (t.hasMoreTokens())
            attrs[j++] = t.nextToken();
        }  
        input1.addElement(str);
        count++;
      }

      if (input1.size() > 0) {

        LDAPSearchResults res = null;

        try {
          
          // now the test case assumes no controls and the server always
          // returns the name and the value of the attribute
          res = ld.search((String)input1.elementAt(0), 
            Integer.parseInt((String)input1.elementAt(1)), 
            (String)input1.elementAt(2), attrs, false, cons);

          int rem = searchCounter - (searchCounter/INTERVAL)*INTERVAL;

          // print the result for every INTERVAL+1
          if ((searchCounter != 0) && (rem == 0))
          {
            date = new Date();
            currTime = date.getTime();
            System.out.println("***** Search #"+searchCounter+" -> " + currTime);
            printResults(ld, res);
          }
        } catch (LDAPException le) {
          System.out.println("Exception: "+le);
        }
      } 
      searchCounter++;
      if (str == null)
        break;
    }
    date = new Date();
    currTime = date.getTime();
    System.out.println("***** finish search -> "+currTime);

    try {
      Thread.sleep(20000);
    } catch (InterruptedException e) {
      System.out.println("exception "+e.toString());
    }

    System.out.println("All finish");

    System.out.println("int num of hits "+cache.getNumHits());
    System.out.println("int num of total operations "+cache.getTotalOperations());
    System.out.println("float num of hits "+((float)cache.getNumHits()));
    System.out.println("float num of total operations "+((float)cache.getTotalOperations()));
    float ratio = (float)(cache.getNumHits())/((float)cache.getTotalOperations());

    System.out.println("hit ratio = "+ratio);
    System.out.println("flush times = "+cache.getNumFlushes());

    try {
      ld.disconnect();
    } catch(LDAPException e) {
      System.out.println("Exception: "+e);
    }
  }

  public static void printResults(LDAPConnection ld, LDAPSearchResults myResults) {
    LDAPEntry myEntry = null;

    System.out.println("printResults");

    while ( myResults.hasMoreElements() ) {
      try {
        myEntry = myResults.next();
      } catch (LDAPReferralException e) { 
        System.out.println("Referral URLs: "); 
        LDAPUrl refUrls[] = e.getURLs(); 
        for (int i = 0; i < refUrls.length; i++) 
          System.out.println("--> "+ refUrls[i].getUrl()); 
        continue; 
      } catch (LDAPException e) { 
        System.out.println("In printResults Exception: "+e); 
        continue; 
      }

      String nextDN = myEntry.getDN();
      System.out.println( nextDN );
      LDAPAttributeSet entryAttrs = myEntry.getAttributeSet();
      Enumeration attrsInSet = entryAttrs.getAttributes();
      while ( attrsInSet.hasMoreElements() ) {
        LDAPAttribute nextAttr = (LDAPAttribute)attrsInSet.nextElement();

        String attrName = nextAttr.getName();
        System.out.println( "\t" + attrName + ":" );
        Enumeration valsInAttr = nextAttr.getStringValues();
        if (valsInAttr != null) {
          while ( valsInAttr.hasMoreElements() ) {
            String nextValue = (String)valsInAttr.nextElement();
            System.out.println( "\t\t" + nextValue );
          }
        }
      }
    }
    System.out.println("finish printResults");
  }
}

