/*
 * @(#) RandomData.java
 *
 * Copyright 1997 Netscape Communications Corp.  All rights reserved.
 */
package netscape.ldap.tests;

import java.util.*;
import java.io.*;
import java.lang.*;

/**
 * This class generates the random data entries for testing the memory cache.
 * Each entry is in the following format:
 * <pre>
 *    o=Airius.com, c=us
 *    2
 *    givenname=David
 *    givenname uid
 *
 *    The first line represents the base DN for search operation, second line is the scope
 *    of the search, the third one is the filter, and the fourth one is the attribute list.
 *    Each entry is separated by the empty line.
 *
 * </pre>
 * 
 * The user provides 2 arguments when running this class. The first one is the file containing
 * different filters. The second one is the file which stores all the random entries. This 
 * program randomly picks a filter for each entry from the filter file which is generated by
 * createFilters.pl. It also randomly picks a baseDN, attribute list for each entry from the 
 * dnTable and attrTable arrays respectively.
 */
public class RandomData {
  private Random rand = new Random(125678123);

  private static String[] dnTable = {
   "o=Airius.com, c=us",
   "ou=People, o=airius.com, c=us",
   "ou=groups,o=airius.com,c=us", 
   "ou=Netscape Servers,o=airius.com,c=us",
   "ou=Product Development, o=airius.com,c=us", 
   "ou=Special Users,o=airius.com,c=us"};

  private static String[] attrTable = {
    "cn", "uid", "uid givenname", "mail", "roomnumber", "ou", "sn","l",
    "dn", "uniquemember", "description", 
    "telephonenumber facsimiletelephonenumber",
    "telephonenumber", "facsimiletelephonenumber"};

  public void genRandomSample(String inputFile, String outputFile) {
    RandomAccessFile iFile = null;
    PrintWriter oFile = null;
    long length = 0;

    try {
      iFile = new RandomAccessFile(inputFile, "r");
      oFile = new PrintWriter(new FileWriter(outputFile));
      length = iFile.length();
      System.out.println("length of the file "+length);
    } catch (IOException e) {
      System.out.println("Exception: "+e);
    }

     
    for (int i=0; i<10000; i++) {
      long num = getNumber(length);
      oFile.println(dnTable[(int)(getNumber(6))]);
      oFile.println((int)(getNumber(3)));
      String str = null;
      try { 
        iFile.seek(num);

        // skip the incomplete line first,since the file pointer might
        // point to the middle of the line. if so, skip it
        str = iFile.readLine();
        
        if (str == null) 
          iFile.seek(0);

        str = iFile.readLine();

        // if this is the end of the file, then we need to reset to the 
        // beginning of the file and read the line again
        if (str == null) 
        {
          iFile.seek(0);
          str = iFile.readLine();
        }

        String subStr = str.substring(0, (str.length())-1);
        oFile.println(subStr);
      } catch (IOException e) {
        System.out.println("Exception: "+e);
      }
      oFile.println(attrTable[(int)(getNumber(14))]);
      oFile.println("");
    }
    oFile.flush();
    oFile.close();
  }

  private long getNumber(long max) {
    long gen = (long)(Math.random()*max);
    long mult = gen/max;
    long rem = gen - (mult*max);
    return rem; 
  }

  public static void main(String[] args) {
    RandomData dummy = new RandomData();
    dummy.genRandomSample(args[0], args[1]);
  }
}

